@@ -5,6 +5,7 @@
 
 from abc import ABC, abstractmethod
 from os import SEEK_SET, stat
+from typing import IO, Iterable, List, Optional, Pattern
 
 from psutil import AccessDenied, NoSuchProcess
 
@@ -14,35 +15,44 @@ __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
 
 
+class _LogContentsCheckState:
+    __slots__ = ("buffer", "fname", "offset")
+
+    def __init__(self, fname: str) -> None:
+        self.fname: str = fname
+        self.buffer: bytes = b""
+        self.offset: int = 0
+
+
 class Check(ABC):
     """
     Check base class
     """
 
-    name = None
+    name: Optional[str] = None
 
     __slots__ = ("message",)
 
-    def __init__(self):
-        self.message = None
+    def __init__(self) -> None:
+        self.message: Optional[str] = None
 
     @abstractmethod
-    def check(self):
+    def check(self) -> Optional[bool]:
         """
         Implement a check that returns True when the abort conditions are met.
         """
 
-    def dump_log(self, dst_fp):
+    def dump_log(self, dst_fp: IO[bytes]) -> None:
         """Write log contents to file.
 
         Args:
-            dst_fp (file): Open file object to write logs to.
+            dst_fp: Open file object to write logs to.
 
         Returns:
             None
         """
         if self.message is not None:
-            dst_fp.write(self.message.encode("utf-8", "ignore"))
+            dst_fp.write(self.message.encode(errors="ignore"))
 
 
 class CheckLogContents(Check):
@@ -56,44 +66,46 @@ class CheckLogContents(Check):
 
     __slots__ = ("logs", "tokens")
 
-    def __init__(self, log_files, search_tokens):
+    def __init__(
+        self, log_files: Iterable[str], search_tokens: Iterable[Pattern[str]]
+    ) -> None:
         assert log_files, "log_files is empty"
         assert search_tokens, "search_tokens is empty"
         super().__init__()
-        self.logs = list()
+        self.logs: List[_LogContentsCheckState] = list()
         for log_file in log_files:
-            self.logs.append({"fname": log_file, "buffer": b"", "offset": 0})
+            self.logs.append(_LogContentsCheckState(log_file))
         self.tokens = search_tokens
 
-    def check(self):
+    def check(self) -> bool:
         """Collect log contents for tokens.
 
         Args:
             None
 
         Returns:
-            bool: True if a token is located otherwise False.
+            True if a token is located otherwise False.
         """
         for log in self.logs:
             try:
                 # check if file has new data
-                if stat(log["fname"]).st_size <= log["offset"]:
+                if stat(log.fname).st_size <= log.offset:
                     continue
-                with open(log["fname"], "rb") as scan_fp:
+                with open(log.fname, "rb") as scan_fp:
                     # only collect new data
-                    scan_fp.seek(log["offset"], SEEK_SET)
+                    scan_fp.seek(log.offset, SEEK_SET)
                     # read and prepend chunk of previously read data
-                    data = b"".join([log["buffer"], scan_fp.read(self.chunk_size)])
-                    log["offset"] = scan_fp.tell()
-            except (IOError, OSError):
+                    data = b"".join((log.buffer, scan_fp.read(self.chunk_size)))
+                    log.offset = scan_fp.tell()
+            except OSError:
                 # log does not exist
                 continue
             for token in self.tokens:
-                match = token.search(data.decode("utf-8", errors="replace"))
+                match = token.search(data.decode(errors="replace"))
                 if match:
-                    self.message = "TOKEN_LOCATED: %s\n" % (match.group(),)
+                    self.message = f"TOKEN_LOCATED: {match.group()}\n"
                     return True
-            log["buffer"] = data[-1 * self.buf_limit :]
+            log.buffer = data[-1 * self.buf_limit :]
         return False
 
 
@@ -106,40 +118,31 @@ class CheckLogSize(Check):
 
     __slots__ = ("limit", "stderr_file", "stdout_file")
 
-    def __init__(self, limit, stderr_file, stdout_file):
+    def __init__(self, limit: int, stderr_file: str, stdout_file: str) -> None:
         super().__init__()
         self.limit = limit
         self.stderr_file = stderr_file
         self.stdout_file = stdout_file
 
-    def check(self):
+    def check(self) -> bool:
         """Collect log disk usage info and compare with limit.
 
         Args:
             None
 
         Returns:
-            bool: True if the total usage is greater than or equal to
-                  self.limit otherwise False.
+            True if the total usage is greater than or equal to
+            self.limit otherwise False.
         """
         err_size = stat(self.stderr_file).st_size
         out_size = stat(self.stdout_file).st_size
         total_size = err_size + out_size
         if total_size > self.limit:
             self.message = (
-                "LOG_SIZE_LIMIT_EXCEEDED: %s\n"
-                "Limit: %s (%dMB)\n"
-                "stderr log: %s (%dMB)\n"
-                "stdout log: %s (%dMB)\n"
-                % (
-                    format(total_size, ","),
-                    format(self.limit, ","),
-                    self.limit / 1048576,
-                    format(err_size, ","),
-                    err_size / 1048576,
-                    format(out_size, ","),
-                    out_size / 1048576,
-                )
+                f"LOG_SIZE_LIMIT_EXCEEDED: {total_size:,}\n"
+                f"Limit: {self.limit:,} ({self.limit / 1_048_576}MB)\n"
+                f"stderr log: {err_size:,} ({err_size / 1_048_576}MB)\n"
+                f"stdout log: {out_size:,} ({out_size / 1_048_576}MB)\n"
             )
         return self.message is not None
 
@@ -154,20 +157,20 @@ class CheckMemoryUsage(Check):
 
     __slots__ = ("limit", "pid")
 
-    def __init__(self, pid, limit):
+    def __init__(self, pid: int, limit: int) -> None:
         super().__init__()
         self.limit = limit
         self.pid = pid
 
-    def check(self):
+    def check(self) -> bool:
         """Use psutil to collect memory usage info and compare with limit.
 
         Args:
             None
 
         Returns:
-            bool: True if the total usage is greater than or equal to
-                  self.limit otherwise False.
+            True if the total usage is greater than or equal to
+            self.limit otherwise False.
         """
         proc_info = list()
         total_usage = 0
@@ -180,11 +183,11 @@ class CheckMemoryUsage(Check):
                 pass
         if total_usage >= self.limit:
             msg = [
-                "MEMORY_LIMIT_EXCEEDED: %s\n" % (format(total_usage, ","),),
-                "Limit: %s (%dMB)\n" % (format(self.limit, ","), self.limit / 1048576),
-                "Parent PID: %d\n" % (self.pid,),
+                f"MEMORY_LIMIT_EXCEEDED: {total_usage:,}\n",
+                f"Limit: {self.limit:,} ({self.limit / 1_048_576}MB)\n",
+                f"Parent PID: {self.pid}\n",
             ]
             for pid, usage in proc_info:
-                msg.append("-> PID %6d: %s\n" % (pid, format(usage, "14,")))
+                msg.append(f"-> PID {pid: 6}: {usage: 14,}\n")
             self.message = "".join(msg)
         return self.message is not None
