@@ -5,7 +5,7 @@
 
 from enum import Enum, unique
 from logging import getLogger
-from os import X_OK, access, getenv
+from os import X_OK, access, getenv, getpid
 from os.path import abspath, dirname, isfile
 from os.path import join as pathjoin
 from os.path import realpath
@@ -26,7 +26,7 @@ try:
 except ImportError:
     pass
 
-from psutil import AccessDenied, NoSuchProcess, wait_procs
+from psutil import AccessDenied, NoSuchProcess, Process, process_iter, wait_procs
 
 try:
     from xvfbwrapper import Xvfb
@@ -39,6 +39,7 @@ from .exceptions import InvalidPrefs, LaunchError, TerminateError
 from .helpers import (
     append_prefs,
     create_profile,
+    files_in_use,
     get_processes,
     onerror,
     prepare_environment,
@@ -84,8 +85,9 @@ class FFPuppet:
 
     Attributes:
         debugger: Debugger to use.
+        headless: Headless mode to use.
         use_profile: Path to existing user profile.
-        use_xvfb: Use Xvfb.
+        use_xvfb: Use Xvfb (DEPRECATED).
         working_path: Path to use as base directory for temporary files.
     """
 
@@ -97,6 +99,7 @@ class FFPuppet:
         "_bin_path",
         "_checks",
         "_dbg",
+        "_headless",
         "_launches",
         "_logs",
         "_proc",
@@ -110,6 +113,7 @@ class FFPuppet:
     def __init__(
         self,
         debugger: Debugger = Debugger.NONE,
+        headless: Optional[str] = None,
         use_profile: Optional[str] = None,
         use_xvfb: bool = False,
         working_path: Optional[str] = None,
@@ -122,6 +126,7 @@ class FFPuppet:
         ] = list()
         self._dbg = debugger
         self._dbg_sanity_check(self._dbg)
+        self._headless = headless
         self._launches = 0  # number of successful browser launches
         self._logs = PuppetLogger(base_path=working_path)
         self._proc: Optional["Popen[bytes]"] = None
@@ -132,15 +137,19 @@ class FFPuppet:
         self.reason: Optional[Reason] = Reason.CLOSED
 
         if use_xvfb:
-            if not system().startswith("Linux"):
-                self._logs.clean_up(ignore_errors=True)
-                raise OSError("Xvfb is only supported on Linux")
+            self._headless = "xvfb"
+
+        if self._headless == "xvfb":
             try:
                 self._xvfb = Xvfb(width=1280, height=1024)
             except NameError:
                 self._logs.clean_up(ignore_errors=True)
-                raise OSError("Please install xvfbwrapper") from None
+                raise OSError(
+                    "Please install xvfbwrapper (Only supported on Linux)"
+                ) from None
             self._xvfb.start()
+        else:
+            assert self._headless in (None, "default")
 
     def __enter__(self) -> "FFPuppet":
         return self
@@ -212,31 +221,22 @@ class FFPuppet:
             Log on the filesystem.
         """
         assert self._logs.working_path is not None
-        try:
-            for entry in Path(self._logs.working_path).iterdir():
-                # scan for sanitizer logs
-                if entry.name.startswith(self._logs.PREFIX_SAN):
-                    if skip_benign and self._benign_sanitizer_report(entry):
-                        continue
+        log_path = Path(self._logs.working_path)
+        # scan for sanitizer logs
+        for entry in log_path.glob(f"{self._logs.PREFIX_SAN}*"):
+            if skip_benign and self._benign_sanitizer_report(entry):
+                continue
+            yield entry.resolve()
+        # scan for Valgrind logs
+        if self._dbg == Debugger.VALGRIND:
+            for entry in log_path.glob(f"{self._logs.PREFIX_VALGRIND}*"):
+                if entry.stat().st_size:
                     yield entry.resolve()
-                # scan for Valgrind logs
-                elif self._dbg == Debugger.VALGRIND and entry.name.startswith(
-                    self._logs.PREFIX_VALGRIND
-                ):
-                    if entry.stat().st_size:
-                        yield entry.resolve()
-        except OSError:  # pragma: no cover
-            pass
-
-        # check for minidumps
+        # scan for minidump files
         if not skip_md:
             assert self.profile is not None
-            try:
-                for entry in (Path(self.profile) / "minidumps").iterdir():
-                    if ".dmp" in entry.name:
-                        yield entry.resolve()
-            except OSError:  # pragma: no cover
-                pass
+            for entry in (Path(self.profile) / "minidumps").glob("*.dmp"):
+                yield entry.resolve()
 
     @classmethod
     def _dbg_sanity_check(cls, dbg: Debugger) -> None:
@@ -277,47 +277,37 @@ class FFPuppet:
                 raise OSError(f"Valgrind >= {cls.VALGRIND_MIN_VERSION:.2f} is required")
 
     @staticmethod
-    def _terminate(pid: int, retry_delay: int = 30, start_mode: int = 0) -> None:
-        """Terminate the process. Each mode (retry pass) is more aggressive.
-        At the end of each attempt if there are active processes mode is
-        incremented and another pass is performed.
-
-        Mode:
-        - 0: uses process.terminate() on the parent process only.
-        - 1: uses process.terminate() on all processes.
-        - 2: uses process.kill() on all processes.
+    def _terminate(
+        procs: List[Process], retry_delay: int = 30, use_kill: bool = False
+    ) -> None:
+        """Call terminate() on provided processes. If terminate() fails try kill().
 
         Args:
+            procs: Processes to terminate/kill.
             retry_delay: Time in seconds to wait before next attempt.
-            start_mode: Initial mode.
+            use_kill: Initial mode.
 
         Returns:
             None
         """
-        LOG.debug(
-            "_terminate(%d, retry_delay=%0.2f, start_mode=%d)",
-            pid,
-            retry_delay,
-            start_mode,
-        )
-        procs = get_processes(pid)
-        for mode in range(start_mode, 3):
-            LOG.debug("%d running process(es)", len(procs))
-            # iterate over and terminate/kill processes
+        while True:
+            LOG.debug(
+                "calling %s on %d running process(es)",
+                "kill()" if use_kill else "terminate()",
+                len(procs),
+            )
+            # iterate over processes and call terminate()/kill()
             for proc in procs:
                 try:
-                    proc.kill() if mode > 1 else proc.terminate()
+                    proc.kill() if use_kill else proc.terminate()
                 except (AccessDenied, NoSuchProcess):  # pragma: no cover
                     pass
-                if mode == 0:
-                    # only target the parent process on the first pass
-                    break
             procs = wait_procs(procs, timeout=retry_delay)[1]
-            if not procs:
-                LOG.debug("_terminate() was successful")
+            if not procs or use_kill:
                 break
-            LOG.debug("timed out (%0.2f), mode %d", retry_delay, mode)
-        else:
+            use_kill = True
+
+        if procs:
             for proc in procs:
                 try:
                     LOG.warning(
@@ -325,7 +315,7 @@ class FFPuppet:
                     )
                 except (AccessDenied, NoSuchProcess):  # pragma: no cover
                     pass
-            raise TerminateError("Failed to terminate browser")
+            raise TerminateError("Failed to terminate processes")
 
     def add_abort_token(self, token: str) -> None:
         """Add a token that when present in the browser log will have the
@@ -369,6 +359,8 @@ class FFPuppet:
         if bin_path.lower().endswith(".py"):
             cmd.append(executable)
         cmd += [bin_path, "-no-remote"]
+        if self._headless == "default":
+            cmd.append("-headless")
         if self.profile is not None:
             cmd += ["-profile", self.profile]
 
@@ -533,40 +525,84 @@ class FFPuppet:
         pid = self.get_pid()
         procs = get_processes(pid) if pid is not None else list()
         LOG.debug("browser pid: %r, %d proc(s)", pid, len(procs))
+
+        # If the parent process closes while other processes are still open
+        # get_processes() will return an empty list, perform a secondary scan if needed
+        if not procs and self._logs.get_fp("stderr"):
+            stderr_fp = self._logs.get_fp("stderr")
+            assert stderr_fp is not None
+            # create a list of processes that are using the stderr file
+            # this *should* only include the browser and the current Python process
+            procs = []
+            for _, other_pid, _ in files_in_use(
+                [Path(stderr_fp.name)], process_iter(["pid", "name", "open_files"])
+            ):
+                # don't include the current Python process in the results
+                if other_pid != getpid():
+                    try:
+                        procs.append(Process(other_pid))
+                    except (AccessDenied, NoSuchProcess):  # pragma: no cover
+                        pass
+            LOG.debug("secondary scan found %d process(es)", len(procs))
+
         # set reason code
-        crash_reports = set(self._crashreports(skip_benign=True))
-        if crash_reports:
+        exit_code = None
+        if any(self._crashreports(skip_benign=True)):
             r_code = Reason.ALERT
-            while True:
-                LOG.debug("%d crash report(s) found", len(crash_reports))
-                # wait until crash report files are closed
-                report_wait = 90 if self._dbg == Debugger.NONE else 300
-                if not wait_on_files(procs, crash_reports, timeout=report_wait):
-                    LOG.warning("Crash reports still open after %ds", report_wait)
-                    break
-                new_reports = set(self._crashreports(skip_benign=True))
-                # verify no new reports have appeared
-                if not new_reports - crash_reports:
-                    break
-                LOG.debug("more reports have appeared")
-                crash_reports = new_reports
+            # Wait a moment for processes to exit automatically.
+            # This will allow crash reports to be fully written to disk.
+            # This assumes a crash report is written and all processes exit
+            # when an issue is detected.
+            # Be sure MOZ_CRASHREPORTER_SHUTDOWN=1 to avoid delays.
+            procs = wait_procs(
+                procs,
+                timeout=30 if self._dbg == Debugger.NONE else 60,
+            )[1]
+            if procs:
+                LOG.warning(
+                    "Slow shutdown detected, %d process(es) still running",
+                    len(procs),
+                )
+            crash_reports = set(self._crashreports(skip_benign=True))
+            LOG.debug("%d crash report(s) found", len(crash_reports))
+            # wait until crash report files are closed
+            report_wait = 60 if self._dbg == Debugger.NONE else 300
+            if not wait_on_files(crash_reports, timeout=report_wait):
+                LOG.warning("Crash reports still open after %ds", report_wait)
+            # get active processes after waiting for crash reports to close
+            procs = wait_procs(procs, timeout=0)[1]
         elif self.is_running():
             r_code = Reason.CLOSED
-        elif self._proc.poll() not in (0, -1, 1, -2):
+            assert procs
+        elif self._proc.poll() not in (0, -1, 1, -2, -9, 245):
+            # Note: ignore 245 for now to avoid getting flooded with OOMs that don't
+            # have a crash report... this should be revisited when time allows
+            # https://bugzil.la/1370520
+            # Ignore -9 to avoid false positives due to system OOM killer
+            exit_code = self._proc.poll()
             r_code = Reason.ALERT
-            LOG.debug("poll() returned %r", self._proc.poll())
+            LOG.warning(
+                "Browser exit code: %r (%X), no crash reports found",
+                exit_code,
+                exit_code,
+            )
         else:
             r_code = Reason.EXITED
+
         # close processes
-        if self.is_running():
-            LOG.debug("browser needs to be terminated")
-            # when running under a debugger be less aggressive
-            assert pid is not None
-            self._terminate(pid, start_mode=1 if self._dbg == Debugger.NONE else 0)
-        # wait for any remaining processes to close
-        if wait_procs(procs, timeout=1 if force_close else 30)[1]:
-            LOG.warning("Some browser processes are still running!")
-        # collect crash logs
+        if procs:
+            # when running under a debugger close the debugger first
+            if self._dbg != Debugger.NONE and self.is_running():
+                LOG.debug("attempting to close debugger")
+                try:
+                    procs[0].terminate()
+                    # only wait if terminate() call does not fail
+                    procs = wait_procs(procs, timeout=10)[1]
+                except (AccessDenied, NoSuchProcess):  # pragma: no cover
+                    pass
+            self._terminate(procs)
+
+        # collect crash reports and logs
         if not force_close:
             if self._logs.closed:  # pragma: no cover
                 # This should not happen while everything is working as expected.
@@ -605,7 +641,10 @@ class FFPuppet:
                     )
                 stderr_fp = self._logs.get_fp("stderr")
                 if stderr_fp:
+                    if exit_code is not None:
+                        stderr_fp.write(f"[ffpuppet] Exit code: {exit_code}\n".encode())
                     stderr_fp.write(f"[ffpuppet] Reason code: {r_code.name}\n".encode())
+
         # reset remaining to closed state
         try:
             self._proc = None
