@@ -345,6 +345,13 @@ def test_ffpuppet_13(mocker):
     )
     # success
     fake_system.return_value = "Linux"
+    with FFPuppet(headless="xvfb"):
+        pass
+    assert fake_xvfb.call_count == 1
+    assert fake_xvfb.return_value.start.call_count == 1
+    fake_xvfb.reset_mock()
+    # success - legacy
+    fake_system.return_value = "Linux"
     with FFPuppet(use_xvfb=True):
         pass
     assert fake_xvfb.call_count == 1
@@ -353,12 +360,7 @@ def test_ffpuppet_13(mocker):
     # not installed
     fake_xvfb.side_effect = NameError
     with raises(OSError, match="Please install xvfbwrapper"):
-        FFPuppet(use_xvfb=True)
-    assert fake_xvfb.start.call_count == 0
-    # unsupported os
-    fake_system.return_value = "Windows"
-    with raises(OSError, match="Xvfb is only supported on Linux"):
-        FFPuppet(use_xvfb=True)
+        FFPuppet(headless="xvfb")
     assert fake_xvfb.start.call_count == 0
 
 
@@ -395,7 +397,11 @@ def test_ffpuppet_14(tmp_path):
 def test_ffpuppet_15(mocker, tmp_path):
     """test launching with gdb"""
     mocker.patch("ffpuppet.core.check_output", autospec=True)
-    mocker.patch("ffpuppet.core.get_processes", autospec=True, return_value=[])
+    mocker.patch(
+        "ffpuppet.core.get_processes",
+        autospec=True,
+        return_value=[mocker.Mock(spec=Process)],
+    )
     mocker.patch("ffpuppet.core.system", autospec=True, return_value="Linux")
     fake_bts = mocker.patch("ffpuppet.core.Bootstrapper", autospec=True)
     fake_bts.return_value.location = "http://test:123"
@@ -429,7 +435,11 @@ def test_ffpuppet_17(mocker, tmp_path):
     mocker.patch(
         "ffpuppet.core.check_output", autospec=True, return_value=b"valgrind-99.0"
     )
-    mocker.patch("ffpuppet.core.get_processes", autospec=True, return_value=[])
+    mocker.patch(
+        "ffpuppet.core.get_processes",
+        autospec=True,
+        return_value=[mocker.Mock(spec=Process)],
+    )
     mocker.patch("ffpuppet.core.system", autospec=True, return_value="Linux")
     fake_bts = mocker.patch("ffpuppet.core.Bootstrapper", autospec=True)
     fake_bts.return_value.location = "http://test:123"
@@ -561,6 +571,8 @@ def test_ffpuppet_22(tmp_path):
                 log_fp.write("filler line\n")
         assert not ffp.is_healthy()
         assert ffp.is_running()
+        # close fake browser process before calling close to avoid hang
+        Process(ffp.get_pid()).terminate()
         ffp.close()
         assert ffp.reason == Reason.ALERT
         logs = tmp_path / "logs"
@@ -604,6 +616,8 @@ def test_ffpuppet_23(mocker, tmp_path):
         (md_path / "test2.dmp").write_text("2a\n2b")
         (md_path / "test3.dmp").write_text("3a\n3b")
         assert not ffp.is_healthy()
+        # close fake browser process before calling close to avoid hang
+        Process(ffp.get_pid()).terminate()
         ffp.close()
         logs = tmp_path / "logs"
         ffp.save_logs(str(logs))
@@ -615,7 +629,11 @@ def test_ffpuppet_23(mocker, tmp_path):
 def test_ffpuppet_24(mocker, tmp_path):
     """test launching with rr"""
     mocker.patch("ffpuppet.core.check_output", autospec=True)
-    mocker.patch("ffpuppet.core.get_processes", autospec=True, return_value=[])
+    mocker.patch(
+        "ffpuppet.core.get_processes",
+        autospec=True,
+        return_value=[mocker.Mock(spec=Process)],
+    )
     mocker.patch("ffpuppet.core.system", autospec=True, return_value="Linux")
     fake_bts = mocker.patch("ffpuppet.core.Bootstrapper", autospec=True)
     fake_bts.return_value.location = "http://test:123"
@@ -751,6 +769,14 @@ def test_ffpuppet_28(tmp_path):
         assert len(cmd) == 3
         assert cmd[0] == "bin_path"
         assert cmd[-1] == "test"
+        assert "-headless" not in cmd
+        # headless
+        ffp._headless = "default"
+        cmd = ffp.build_launch_cmd("bin_path", ["test"])
+        assert len(cmd) == 4
+        assert cmd[0] == "bin_path"
+        assert cmd[-1] == "test"
+        assert "-headless" in cmd
         # GDB
         ffp._dbg = Debugger.GDB
         cmd = ffp.build_launch_cmd("bin_path")
@@ -864,37 +890,28 @@ def test_ffpuppet_31(mocker):
         mocker.Mock(spec_set=Process, pid=123),
         mocker.Mock(spec_set=Process, pid=124),
     ]
-    mocker.patch("ffpuppet.core.get_processes", autospec=True, return_value=procs)
+    mocker.patch("ffpuppet.core.get_processes", autospec=True)
     fake_wait_procs = mocker.patch("ffpuppet.core.wait_procs", autospec=True)
-    # successful call to terminate (parent process only)
-    fake_wait_procs.return_value = ([], [])
-    FFPuppet._terminate(1234)
-    assert sum(x.terminate.call_count for x in procs) == 1
-    assert not sum(x.kill.call_count for x in procs)
-    assert procs[0].terminate.call_count == 1
-    for proc in procs:
-        proc.reset_mock()
-    # successful call to terminate (all processes)
-    fake_wait_procs.return_value = None
-    fake_wait_procs.side_effect = (([], [procs[-1]]), ([], []))
-    FFPuppet._terminate(1234)
+    # successful call to terminate
+    fake_wait_procs.side_effect = (([], []),)
+    FFPuppet._terminate(procs)
     assert sum(x.terminate.call_count for x in procs) == 2
     assert not sum(x.kill.call_count for x in procs)
     for proc in procs:
         proc.reset_mock()
     # successful call to kill
-    fake_wait_procs.return_value = None
-    fake_wait_procs.side_effect = (([], procs), ([], procs), ([], []))
-    FFPuppet._terminate(1234)
-    assert sum(x.terminate.call_count for x in procs) == 3
+    fake_wait_procs.side_effect = (([], procs), ([], []))
+    FFPuppet._terminate(procs)
+    assert sum(x.terminate.call_count for x in procs) == 2
     assert sum(x.kill.call_count for x in procs) == 2
     for proc in procs:
         proc.reset_mock()
     # failed call to kill
-    fake_wait_procs.return_value = ([], procs)
-    fake_wait_procs.side_effect = None
+    fake_wait_procs.side_effect = (([], procs), ([], procs))
     with raises(TerminateError):
-        FFPuppet._terminate(1234)
+        FFPuppet._terminate(procs)
+    assert sum(x.terminate.call_count for x in procs) == 2
+    assert sum(x.kill.call_count for x in procs) == 2
 
 
 def test_ffpuppet_32(mocker, tmp_path):
@@ -906,22 +923,27 @@ def test_ffpuppet_32(mocker, tmp_path):
             self.reason = None
             self._bin_path = str(tmp_path)
             self._logs.reset()
+            self._logs.add_log("stderr")
             self._proc = mocker.Mock(spec=Popen, pid=123)
             self._proc.poll.return_value = None
             profile = tmp_path / "profile"
             profile.mkdir(exist_ok=True)
             self.profile = str(profile)
 
-        @staticmethod
-        def _terminate(pid, *_a, **_kw):  # pylint: disable=signature-differs
-            assert isinstance(pid, int)
+        def _terminate(self, _procs, _retry_delay=0, _use_kill=False):
+            pass
 
+    mocker.patch(
+        "ffpuppet.core.get_processes",
+        autospec=True,
+        return_value=[mocker.Mock(spec_set=Process)],
+    )
     fake_reports = mocker.patch("ffpuppet.core.FFPuppet._crashreports", autospec=True)
     fake_reports.return_value = ()
     fake_wait_files = mocker.patch("ffpuppet.core.wait_on_files", autospec=True)
-    fake_wait_files.return_value = True
-    mocker.patch("ffpuppet.core.wait_procs", autospec=True)
+    fake_wait_procs = mocker.patch("ffpuppet.core.wait_procs", autospec=True)
     # process exited - no crash
+    fake_wait_procs.side_effect = (([], []),)
     with StubbedProc() as ffp:
         ffp.launch()
         ffp._proc.poll.return_value = 0
@@ -930,6 +952,7 @@ def test_ffpuppet_32(mocker, tmp_path):
         assert ffp._logs.closed
         assert ffp.reason == Reason.EXITED
     # process exited - exit code - crash
+    fake_wait_procs.side_effect = (([], []),)
     with StubbedProc() as ffp:
         ffp.launch()
         ffp._proc.poll.return_value = -11
@@ -938,6 +961,7 @@ def test_ffpuppet_32(mocker, tmp_path):
         assert ffp._logs.closed
         assert ffp.reason == Reason.ALERT
     # process running - no crash reports
+    fake_wait_procs.side_effect = (([], [mocker.Mock()]),)
     with StubbedProc() as ffp:
         ffp.launch()
         ffp._proc.poll.return_value = None
@@ -946,6 +970,7 @@ def test_ffpuppet_32(mocker, tmp_path):
         assert ffp._logs.closed
         assert ffp.reason == Reason.CLOSED
     # process running - with crash reports, hang waiting to close
+    fake_wait_procs.side_effect = (([], [mocker.Mock()]), ([], []))
     (tmp_path / "fake_report1").touch()
     with StubbedProc() as ffp:
         ffp.launch()
@@ -957,6 +982,7 @@ def test_ffpuppet_32(mocker, tmp_path):
         assert ffp._logs.closed
         assert ffp.reason == Reason.ALERT
     # process running - with crash reports, multiple logs
+    fake_wait_procs.side_effect = (([], []), ([], []))
     (tmp_path / "fake_report2").touch()
     with StubbedProc() as ffp:
         ffp.launch()
@@ -966,8 +992,6 @@ def test_ffpuppet_32(mocker, tmp_path):
             (tmp_path / "fake_report1",),
             (tmp_path / "fake_report1", tmp_path / "fake_report2"),
             (tmp_path / "fake_report1", tmp_path / "fake_report2"),
-            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
-            (tmp_path / "fake_report1", tmp_path / "fake_report2"),
         )
         fake_wait_files.return_value = True
         ffp.close()
@@ -990,3 +1014,11 @@ def test_ffpuppet_33():
         assert san_log in ffp._logs.watching
         ffp.close()
         assert ffp.reason == Reason.CLOSED
+
+
+def test_ffpuppet_34(mocker):
+    """test secondary process lookup scan in close()"""
+    fake_get_proc = mocker.patch("ffpuppet.core.get_processes", return_value=[])
+    with FFPuppet() as ffp:
+        ffp.launch(TESTFF_BIN)
+    assert fake_get_proc.call_count == 1
