@@ -9,7 +9,6 @@ from multiprocessing import Event, Process
 from pathlib import Path
 from platform import system
 from shutil import rmtree
-from tempfile import NamedTemporaryFile
 
 from psutil import Process as PSProcess
 from pytest import raises
@@ -362,22 +361,30 @@ def test_helpers_06():
     assert "MOZ_CRASHREPORTER" not in env
 
 
-def test_helpers_07(tmp_path):
+def test_helpers_07(mocker, tmp_path):
     """test wait_on_files()"""
+    fake_sleep = mocker.patch("ffpuppet.helpers.sleep", autospec=True)
+    fake_time = mocker.patch("ffpuppet.helpers.time", autospec=True)
     t_file = tmp_path / "file.bin"
     t_file.touch()
-    # test with open file
-    procs = get_processes(os.getpid(), recursive=False)
-    with NamedTemporaryFile() as wait_fp:
-        assert not wait_on_files(procs, [Path(wait_fp.name), t_file], timeout=0.1)
+    # test with open file (timeout)
+    fake_time.side_effect = (1, 1, 2)
+    with (tmp_path / "open.bin").open("w") as wait_fp:
+        assert not wait_on_files([Path(wait_fp.name), t_file], timeout=0.1)
+    assert fake_sleep.call_count == 1
+    fake_sleep.reset_mock()
     # existing but closed file
-    procs = get_processes(os.getpid(), recursive=False)
-    assert wait_on_files(procs, [t_file], timeout=0.1)
+    fake_time.side_effect = (1, 1)
+    assert wait_on_files([t_file])
+    assert fake_sleep.call_count == 0
     # file that does not exist
-    procs = get_processes(os.getpid(), recursive=False)
-    assert wait_on_files(procs, [Path("missing")], timeout=0.1)
+    fake_time.side_effect = (1, 1)
+    assert wait_on_files([Path("missing")])
+    assert fake_sleep.call_count == 0
     # empty file list
-    assert wait_on_files([], [])
+    fake_time.side_effect = (1, 1)
+    assert wait_on_files([])
+    assert fake_sleep.call_count == 0
 
 
 # this needs to be here in order to work correctly on Windows
@@ -422,7 +429,7 @@ def test_helpers_10(tmp_path):
     t_file.touch()
     procs = [PSProcess(os.getpid())]
     # test with open file
-    with NamedTemporaryFile() as wait_fp:
+    with (tmp_path / "file").open("w") as wait_fp:
         assert any(files_in_use([t_file, Path(wait_fp.name)], procs))
     # existing but closed file
     assert not any(files_in_use([t_file], procs))
@@ -430,6 +437,5 @@ def test_helpers_10(tmp_path):
 
 def test_helpers_11(tmp_path):
     """test warn_open()"""
-    (tmp_path / "file.bin").touch()
-    with NamedTemporaryFile(dir=str(tmp_path)) as _:
+    with (tmp_path / "file.bin").open("w") as _:
         warn_open(str(tmp_path))
